/**
 * Author: Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */

package org.objectweb.fractal.bf;

import java.util.logging.Logger;

/**
 * Extend the basic service with more complex operations, using non primitive
 * types.
 */
public class ComplexServiceImpl implements ComplexService, Runnable {
	Logger log = Logger.getLogger(ComplexServiceImpl.class.getCanonicalName());

	/**
	 * @see org.objectweb.fractal.bf.ComplexService#getProduct()
	 */
	public Product getProduct() {
		Product p = makeProduct();
		log.info("Returning " + p);

		return p;
	}

	/**
	 * @return
	 */
	private Product makeProduct() {
		Product p = new Product();
		p.setId(666);
		p.setName("base-product");

		return p;
	}

	/**
	 * @see org.objectweb.fractal.bf.ComplexService#getCompanionProductOf(org.objectweb.fractal.bf.Product)
	 */
	public Product getCompanionProductOf(Product p) {
		Product comp = makeProduct();
		comp.setName("companion-of-" + p.getName());
		comp.setId(p.getId() + 1);
		log.info("Returning " + p);

		return comp;
	}

	/**
	 * @see org.objectweb.fractal.bf.ComplexService#getProductById(int)
	 */
	public Product getProductById(int i) {
		log.fine("Call to getProductId received on server. Id: " + i);
		Product p = makeProduct();
		p.setId(i);
		log.info("Returning " + p);
		return p;
	}

	/**
	 * @see org.objectweb.fractal.bf.ComplexService#isAvailable(org.objectweb.fractal.bf.Product)
	 */
	public boolean isAvailable(Product p) {
		log.info("Returning true");
		return true;
	}

	/**
	 * @see org.objectweb.fractal.bf.ComplexService#makerOf(org.objectweb.fractal.bf.Product)
	 */
	public ProductMaker makerOf(Product p) {

		final ProductMaker productMaker = new ProductMaker();
		log.info("Returning: " + productMaker);
		return productMaker;
	}

	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		log.info("server is running");

	}

}
